"""
CV-ArtSmartImport v2.94
Copyright: 2012-2013 Rick Barrett
Written for CINEMA 4D R13+
"""

import os
import platform
import ctypes
import webbrowser

import c4d



###############################################################################
#   LOCALIMPORT CLASS FOR CUSTOM LIBRARY IMPORT
###############################################################################
# Copyright (C) 2015  Niklas Rosenstein
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# Uncompressed source code: https://github.com/NiklasRosenstein/localimport

exec("""
#__author__='Niklas Rosenstein <rosensteinniklas@gmail.com>'
#__version__='1.4.3'
import glob,os,pkgutil,sys,traceback
class _localimport(object):
 _py3k=sys.version_info[0]>=3
 _string_types=(str,)if _py3k else(str,)
 def __init__(self,path,parent_dir=os.path.dirname(__file__)):
  super(_localimport,self).__init__()
  self.path=[]
  if isinstance(path,self._string_types):
   path=[path]
  for path_name in path:
   if not os.path.isabs(path_name):
    path_name=os.path.join(parent_dir,path_name)
   self.path.append(path_name)
   self.path.extend(glob.glob(os.path.join(path_name,'*.egg')))
   for fn in glob.glob(os.path.join(path_name,'*.pth')):
    self._eval_pth(fn)
  self.meta_path=[]
  self.modules={}
  self.in_context=False
 def __enter__(self):
  try:import pkg_resources;nsdict=pkg_resources._namespace_packages.copy()
  except ImportError:nsdict=None
  self.state={'nsdict':nsdict,'nspaths':{},'path':sys.path[:],'meta_path':sys.meta_path[:],'disables':{},}
  sys.path[:]=self.path+sys.path
  sys.meta_path[:]=self.meta_path+sys.meta_path
  for key,mod in list(self.modules.items()):
   try:self.state['disables'][key]=sys.modules.pop(key)
   except KeyError:pass
   sys.modules[key]=mod
  for key,mod in list(sys.modules.items()):
   if hasattr(mod,'__path__'):
    self.state['nspaths'][key]=mod.__path__[:]
    mod.__path__=pkgutil.extend_path(mod.__path__,mod.__name__)
  self.in_context=True
  return self
 def __exit__(self,*__):
  if not self.in_context:
   raise RuntimeError('context not entered')
  for key,path in list(self.state['nspaths'].items()):
   sys.modules[key].__path__=path
  for meta in sys.meta_path:
   if meta is not self and meta not in self.state['meta_path']:
    if meta not in self.meta_path:
     self.meta_path.append(meta)
  modules=sys.modules.copy()
  for key,mod in list(modules.items()):
   force_pop=False
   filename=getattr(mod,'__file__',None)
   if not filename and key not in sys.builtin_module_names:
    parent=key.rsplit('.',1)[0]
    if parent in modules:
     filename=getattr(modules[parent],'__file__',None)
    else:
     force_pop=True
   if force_pop or(filename and self._is_local(filename)):
    self.modules[key]=sys.modules.pop(key)
  sys.modules.update(self.state['disables'])
  sys.path[:]=self.state['path']
  sys.meta_path[:]=self.state['meta_path']
  try:
   import pkg_resources
   pkg_resources._namespace_packages.clear()
   pkg_resources._namespace_packages.update(self.state['nsdict'])
  except ImportError:pass
  self.in_context=False
  del self.state
 def _is_local(self,filename):
  filename=os.path.abspath(filename)
  for path_name in self.path:
   path_name=os.path.abspath(path_name)
   if self._is_subpath(filename,path_name):
    return True
  return False
 def _eval_pth(self,filename):
  if not os.path.isfile(filename):
   return
  with open(filename,'r')as fp:
   for index,line in enumerate(fp):
    if line.startswith('import'):
     line_fn='{0}#{1}'.format(filename,index+1)
     try:
      exec(compile(line,line_fn,'exec'))
     except BaseException:
      traceback.print_exc()
    else:
     index=line.find('#')
     if index>0:line=line[:index]
     line=line.strip()
     if not os.path.isabs(line):
      line=os.path.join(os.path.dirname(filename),line)
     line=os.path.normpath(line)
     if line and line not in self.path:
      self.path.append(line)
 @staticmethod
 def _is_subpath(path,ask_dir):
  try:
   relpath=os.path.relpath(path,ask_dir)
  except ValueError:
   return False
  return relpath==os.curdir or not relpath.startswith(os.pardir)
""")

# Import your stuff, for example:
with _localimport('lib') as importer:
    import vecfile, artsmartcore, clipboardwin


#=====================================================================================================================#
#   SYMBOLS
#=====================================================================================================================#
try:
    import symbol_parser
    c4d_symbol_file = os.path.join(os.path.dirname(__file__), "res")
    symbol_parser.parse_and_export_in_caller(c4d_symbol_file)
except ImportError:
    ART_FILE		=30000
    ART_LAYEROFFSET		=30001
    ART_DEPTH		=30002
    ART_CONTENTS		=30003
    ART_RECENTER		=30004
    ART_PATHOFFSET		=30005
    ART_USEMOGRAPH		=30006
    ART_CINEWARE		=30007
    ART_FILTERGROUP		=30010
    ART_LAYERFILTER		=30011
    ART_PATHFILTER_ON	=30012
    ART_PATHFILTER		=30013
    ART_RECENTER_Z		=30014
    ART_FORCEPDF		=30015
    ART_STROKESETTINGS	=30016
    ART_STROKESCALE		=30017
    ART_STROKEASPECT	=30018
    ART_STROKEROUNDING	=30019
    ART_STROKEGROWTH	=30020
    ART_STROKEALIGN		=30021
    ART_STROKEALIGN_START = 30022
    ART_STROKEALIGN_END = 30023
    ART_STROKEALIGN_CENTER = 30024
    ART_STROKEALIGN_STARTEND = 30025
    ART_STROKEOFFSET = 30026
    ART_GREEDYFILL = 30050
    ART_AUTOCLOSE = 30051
    ART_LEGACYCENTER = 30052
    ART_PATHEFFECTOR	=30100
    ART_PATHEFFECTOR_LIST = 30101
    ART_LAYEREFFECTOR   =30200
    ART_LAYEREFFECTOR_LIST = 30201
    ART_RELOAD		=5000
    ART_OPEN		=5001
    ART_CREATEMATERIALS		=5002
    ART_INFO		=5005
    ART_DEBUGMSG	=5006
    ART_CREATEOBJECT = 6000
    ART_CHECKSUM		=9000
    ART_STROKED			=9001
    ART_FILLED			=9002
    ART_STROKECOLOR		=9003
    ART_FILLCOLOR		=9004
    ART_STROKEWIDTH		=9005


# be sure to use a unique ID obtained from www.plugincafe.com
PLUGIN_ID = 1028656
TAG_ID = 1028660
ICON_ID = 1028659
OBJECT_ID = 1028655

DEBUG = False

class ArtSmartOptionDlg(c4d.gui.GeDialog):
       
    def __init__(self):
        self.dlgData = c4d.BaseContainer()
        self.result = c4d.DLG_CANCEL
       
    def CreateLayout(self):
    
        buttonBC = c4d.BaseContainer()
        buttonBC.SetLong(c4d.BITMAPBUTTON_ICONID1,ICON_ID)
                
        self.SetTitle("CV-ArtSmart Options")
        self.AddCustomGui(ART_INFO,c4d.CUSTOMGUI_BITMAPBUTTON, "CV-ArtSmart", c4d.BFH_CENTER, 300, 40, buttonBC)
        self.GroupBegin(0,c4d.BFH_SCALEFIT,2)
        self.AddStaticText(0,c4d.BFH_LEFT,name="File")
        self.AddCustomGui(ART_FILE,c4d.CUSTOMGUI_FILENAME, "File", c4d.BFH_SCALEFIT, 300, 15)
        self.GroupEnd()
        self.AddCheckbox(ART_FORCEPDF, c4d.BFH_LEFT, 300, 12, name="Force PDF")
        self.AddCheckbox(ART_GREEDYFILL, c4d.BFH_LEFT, 300, 12, name="Greedy Fill")
        self.AddCheckbox(ART_RECENTER, c4d.BFH_LEFT, 300, 12, name="Center Artwork")
        self.AddCheckbox(ART_CREATEOBJECT, c4d.BFH_LEFT, 300, 12, name="Create Object")
        self.AddDlgGroup(c4d.DLG_OK|c4d.DLG_CANCEL)
        
        return True
    
    def InitValues(self):
        if self.dlgData:
            self.SetBool(ART_FORCEPDF,self.dlgData.GetBool(ART_FORCEPDF))
            self.SetBool(ART_GREEDYFILL,self.dlgData.GetBool(ART_GREEDYFILL))
            self.SetBool(ART_RECENTER,self.dlgData.GetBool(ART_RECENTER))
            self.SetBool(ART_CREATEOBJECT,self.dlgData.GetBool(ART_CREATEOBJECT))
        return True
    
    def Command(self,id,msg):
        if id == ART_INFO:
            webbrowser.open_new_tab('http://www.cineversity.com/vidplaylist/cv-artsmart/artsmart_download')
        elif id == c4d.DLG_OK:
            self.dlgData.SetData(ART_FILE,self.GetString(ART_FILE))
            self.dlgData.SetBool(ART_FORCEPDF,self.GetBool(ART_FORCEPDF))
            self.dlgData.SetBool(ART_GREEDYFILL,self.GetBool(ART_GREEDYFILL))
            self.dlgData.SetBool(ART_RECENTER,self.GetBool(ART_RECENTER))
            self.dlgData.SetBool(ART_CREATEOBJECT,self.GetBool(ART_CREATEOBJECT))
            self.result = c4d.DLG_OK
            self.Close()
        elif id == c4d.DLG_CANCEL:
            self.Close()
        return True
    
# CommandData class
class ArtSmart(c4d.plugins.CommandData):         

    def __init__(self, pasteMode=False):
        self.pasteMode = pasteMode
       
    def GetClipData(self):
            if os.name == 'nt' or platform.system() == 'Windows':
                aiformat = artsmartcore.FindAIClipFormat()
                if aiformat == 0:
                    print("No Illustrator formatted art found on clipboard")
                    return False

                copy, paste = clipboardwin.init_windows_clipboard()
                text = paste(aiformat)

                return text

            elif os.name == 'mac' or platform.system() == 'Darwin':
                import subprocess
                dir, file = os.path.split(__file__)
                proc = subprocess.Popen([os.path.join(dir, 'clipAICBHandler.app', 'Contents', 'MacOS', 'clipAICBHandler'),'-paste'], stdout=subprocess.PIPE)
                content, err = proc.communicate()
                return content
    
    def ExecuteOptionID(self,doc,plugid,subid):
        dlg = ArtSmartOptionDlg()
        dlg.artsmart = self
        prefData = c4d.plugins.GetWorldPluginData(PLUGIN_ID)
        if prefData:
            dlg.dlgData = prefData
        
        dlg.Open(c4d.DLG_TYPE_MODAL, PLUGIN_ID)
        
        if dlg.result != c4d.DLG_OK: return False
        
        self.Execute(doc,dlg.dlgData)
        
        #Strip the filename - don't want to save that in pref
        dlg.dlgData.SetString(ART_FILE,"")
        
        #Save other options as prefs
        c4d.plugins.SetWorldPluginData(PLUGIN_ID, dlg.dlgData)
        
        return True
    
    def Execute(self, doc, data=None):
             
        art = None
        artsmartcore.checksums = {}
        name = "CV-ArtSmart"    #default name for top object
        center = False
        
        if not data:
            data = c4d.plugins.GetWorldPluginData(PLUGIN_ID)
        
        if data:
            center = data.GetBool(ART_RECENTER)
        
        if self.pasteMode:
            clip = self.GetClipData()
            if DEBUG:
                if clip:
                    f = open(os.path.join(c4d.storage.GeGetStartupWritePath(),'ArtSmartDebug.txt'),'w')
                    f.write(clip)
                    f.close
                else:
                    print("Clipboard Data Empty")
            if clip:
                art = vecfile.ParseVecString(clip)
            else:
                # Fall back to C4D Paste
                c4d.CallCommand(12108) # Paste        
        else:
            
            if data and data.GetString(ART_FILE)!="":
                fn = data.GetData(ART_FILE)
            else:
                fn = c4d.storage.LoadDialog()
            
            if not fn: return False
            
            forcePdf = False
            greedyFill = False
            if data:
                forcePdf = data.GetBool(ART_FORCEPDF)
                greedyFill = data.GetBool(ART_GREEDYFILL)
                
            if data and data.GetBool(ART_CREATEOBJECT):
                artsmartObj = c4d.BaseObject(OBJECT_ID)
                artsmartData = artsmartObj.GetDataInstance()   #have to use BC for the filename
                artsmartData.SetBool(ART_FORCEPDF,forcePdf)
                artsmartData.SetBool(ART_GREEDYFILL,greedyFill)
                artsmartData.SetFilename(ART_FILE,fn)
                artsmartObj.Message(c4d.MSG_CHANGE)
                doc.InsertObject(artsmartObj)
                doc.SetSelection(artsmartObj,c4d.SELECTION_NEW)
                c4d.EventAdd()
                return True
                        
            name = os.path.basename(fn)
            if fn:
                art = vecfile.ParseVecFile(fn,not forcePdf,greedyFill)
                
        return self.LoadArt(doc,art,name,center)

    def GetConstructionPlane(self,doc,returnSigned=False,returnVector=False):

        bd = doc.GetActiveBaseDraw()

        if bd.GetProjection() == 2:     #Left
            axis = "X+"
        elif bd.GetProjection() == 3:   #Right
            axis = "X-"
        elif bd.GetProjection() == 4:   #Front
            axis = "Z+"
        elif bd.GetProjection() == 5:   #Back
            axis = "Z-"
        elif bd.GetProjection() == 6:   #Top
            axis = "Y+"
        elif bd.GetProjection() == 7:   #Bottom
            axis = "Y-"

        else:

            if bd.HasCameraLink():
                cam = bd.GetSceneCamera(doc)
            else:
                cam = bd.GetEditorCamera()

            hpb = c4d.utils.MatrixToHPB(cam.GetMg())
            x = c4d.utils.Deg(hpb.x)

            if c4d.utils.Deg(hpb.y) > 60:
                axis = "Y+"
            elif c4d.utils.Deg(hpb.y) < -60:
                axis = "Y-"
            elif x >= 45 and x <= 135:
                axis = "X-"
            elif x >= 235 and x <= 305:
                axis = "X+"
            elif x > 135 and x < 235:
                axis = "Z-"
            else:
                axis = "Z+"

        if returnVector:
            vec = c4d.Vector(1,0,0)
            if axis[0] == "Y": vec = c4d.Vector(0,1,0)
            elif axis[0] == "Z": vec = c4d.Vector(0,0,1)
            if returnSigned and axis[1]=="-":
                vec = vec * -1
            return vec
        elif not returnSigned:
            return axis[0]
        else:
            return axis

    def LoadArt(self,doc,art,name=None,center=False):

        if not art: return False

        doc.StartUndo()

        #Get Facing Axis

        axis = self.GetConstructionPlane(doc)

        null = c4d.BaseObject(c4d.Onull)
        null.SetName(name)
        null.ChangeNBit(c4d.NBIT_OM1_FOLD,c4d.NBITCONTROL_SET)
        doc.InsertObject(null)
        doc.AddUndo(c4d.UNDOTYPE_NEWOBJ,null)
        
        lyrTop = c4d.documents.LayerObject()
        lyrTop.SetName(name)
        lyrTop.InsertUnder(doc.GetLayerObjectRoot())
        null.SetLayerObject(lyrTop)
        doc.AddUndo(c4d.UNDOTYPE_NEWOBJ,lyrTop)
        
        if art.layers:
            for item in art.layers:
                artsmartcore.ParseLayer(item,null,lyrTop,c4d.Onull,axis=axis)

        else:
            layers = {}
            if art.res and "Properties" in art.res:
                for mc,mcd in art.res["Properties"].items():
                    lyr = c4d.documents.LayerObject()
                    lyr.SetName(mcd[1])
                    lyr.InsertUnder(lyrTop)
                    layers[mc] = lyr
            
            for path in art.paths:
                spl,depth,offset,spread = artsmartcore.CreatePath(path,axis=axis)
                spl.InsertUnder(null)
                if path.markers and path.markers[-1] in layers:
                    spl.SetLayerObject(layers[path.markers[-1]])

        if center:
            self.RecenterArt(null)

        doc.EndUndo()

        c4d.EventAdd()
            
        return True
        
    def RecenterArt(self,op):
        centers = []
        for obj in op.GetChildren():
            centers.append(obj.GetAbsPos())
        if centers:
            if op[ART_LEGACYCENTER]:
                center = sum(centers) / len(centers)
            else:
                center = artsmartcore.FindCenter(centers)
            for obj in op.GetChildren():
                newpos = obj.GetAbsPos()
                newpos.x = newpos.x-center.x
                newpos.y = newpos.y-center.y
                newpos.z = newpos.z-center.z
                obj.SetAbsPos(newpos)

class ArtSmartCopy(c4d.plugins.CommandData):         

    def SetClipData(self,text):
        if os.name == 'nt' or platform.system() == 'Windows':
            aiformat = artsmartcore.FindAIClipFormat()
            if aiformat == 0:
                c4d.gui.MessageDialog("Couldn't find Illustrator Clipboard ID. Please copy art to the clipboard in Illustrator and try again.")
                return False        

            copy, paste = clipboardwin.init_windows_clipboard()
            copy(aiformat, text)
            try:
                copy(aiformat, text)
            except:
                c4d.gui.MessageDialog("Error setting clipboard")

            return text

        elif os.name == 'mac' or platform.system() == 'Darwin':
            import subprocess
            dir, file = os.path.split(__file__)
            proc = subprocess.Popen([os.path.join(dir, 'clipAICBHandler.app', 'Contents', 'MacOS', 'clipAICBHandler'),'-copy',text], stdout=subprocess.PIPE)
            content, err = proc.communicate()
            return content

    def Execute(self, doc):
        
        eps = ""
        
        #loop through selected objects and write each spline
        for op in reversed(doc.GetSelection()):
            eps += artsmartcore.WriteSpline(op)

        if DEBUG:
            f = open(os.path.join(c4d.storage.GeGetStartupWritePath(),'ArtSmartDebug.txt'),'w')
            f.write(eps)
            f.close
            
        if eps == "":
            c4d.gui.MessageDialog("You must select at least one spline type object")
        else:
            self.SetClipData(artsmartcore.AICBHeader()+eps+ artsmartcore.AICBFooter())
                
        c4d.EventAdd()
            
        return True

class ArtSmartTag(c4d.plugins.TagData):

    def Init(self, node):
        #load attributes from .h into c4d. .res describes the structure
        self.InitAttr(host=node, type=str, id=[ART_CHECKSUM])
        self.InitAttr(host=node, type=bool, id=[ART_STROKED])
        self.InitAttr(host=node, type=bool, id=[ART_FILLED])
        self.InitAttr(host=node, type=c4d.Vector, id=[ART_STROKECOLOR])
        self.InitAttr(host=node, type=c4d.Vector, id=[ART_FILLCOLOR])
        self.InitAttr(host=node, type=float, id=[ART_STROKEWIDTH])
        
        node[ART_CHECKSUM] = ""
        node[ART_STROKED] = False
        node[ART_FILLED] = False
        node[ART_STROKECOLOR] = c4d.Vector(0,0,0)
        node[ART_FILLCOLOR] = c4d.Vector(0,0,0)
        node[ART_STROKEWIDTH] = 0
        
        return True
        
# main
if __name__ == "__main__":
    # load icon.tif from res into bmp
    dir, file = os.path.split(__file__)
    fn = os.path.join(dir, "res", "icon.psd")
    fnCopy = os.path.join(dir, "res", "icon-copy.psd")
    fnPaste = os.path.join(dir, "res", "icon-paste.psd")
    bmp = c4d.bitmaps.BaseBitmap()
    bmp.InitWith(fn)
    bmpCopy = c4d.bitmaps.BaseBitmap()
    bmpCopy.InitWith(fnCopy)
    bmpPaste = c4d.bitmaps.BaseBitmap()
    bmpPaste.InitWith(fnPaste)
    # register the plugin
    c4d.plugins.RegisterCommandPlugin(id=PLUGIN_ID, 
                                  str="CV-ArtSmart Import",
                                  info=c4d.PLUGINFLAG_COMMAND_OPTION_DIALOG,
                                  help="Imports Illustrator and PDF splines", 
                                  dat=ArtSmart(False),
                                  icon=bmp)
    c4d.plugins.RegisterCommandPlugin(id=PLUGIN_ID+1, 
                                  str="CV-ArtSmart Copy",
                                  info=0,
                                  help="Copies Illustrator and PDF splines to the Clipboard", 
                                  dat=ArtSmartCopy(),
                                  icon=bmpCopy)
    c4d.plugins.RegisterCommandPlugin(id=PLUGIN_ID+2, 
                                  str="CV-ArtSmart Paste",
                                  info=0,
                                  help="Pastes Illustrator and PDF splines from the Clipboard", 
                                  dat=ArtSmart(True),
                                  icon=bmpPaste)
    c4d.plugins.RegisterTagPlugin(id=TAG_ID,
                                  str="CV-ArtSmart PathInfo",
                                  g=ArtSmartTag,
                                  description="Tcvartsmart",
                                  info=c4d.TAG_VISIBLE|c4d.PLUGINFLAG_HIDE|c4d.PLUGINFLAG_HIDEPLUGINMENU,
                                  icon=bmp)
